%   Script BSPBESCH.M
%
%  Modell einer Beschichtungsanlage
%
% J. Lunze
% 21.6.2002
% fr 2. Auflage: 27.12.2006
% fr 3. Auflage: 1.12.2011
% fr 5. Auflage: 25.10.2019
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   T1 = Temperatur der Deckschicht
%   T2 = Temperatur des Trgermaterials
%   T3 = Temperatur des Werkstckhalters
%
%   u = Wrmestrom auf die Deckschicht
%   y = Temperatur T1 des Werkstckhalters
%
%
%
%  Deckschicht aus Polyamid
d1=0.001;          % m
lambda1=30;        % W/m/K
rho1=1.12*1000;    % kg/m^3
c1=500;            % J/kg/K
%  Trgermaterial aus Aluminium
d2=0.003;          % m
lambda2=221;       % W/m/K
rho2=2.7*1000;     % kg/m^3
c2=921;            % J/kg/K
%  Werkstckhalter aus Eisen
d3=0.005;          % m
lambda3=67;        % W/m/K
rho3=7.87*1000;    % kg/m^3
c3=465;            % J/kg/K
%
Am= 0.1*0.1*pi;      % m^2
%  Am ist bereits aus den Elemente von A gekrzt und erscheint deshalb
%  nicht in den Koeffizienten k21, k32, ku3
%
%  Zwischengren
k21=1/(d1/2/lambda1 + d2/2/lambda2);
k32=1/(d2/2/lambda2 + d3/2/lambda3);
ku3=1/(d3/2/lambda3);
%
%  Zustandsraummodell
A = [-k21/d1/rho1/c1   k21/d1/rho1/c1             0;
    k21/d2/rho2/c2   -(k32+k21)/d2/rho2/c2   k32/d2/rho2/c2
    0                 k32/d3/rho3/c3         -(k32+ku3)/d3/rho3/c3];
B=[1/d1/Am/rho1/c1; 0; 0];
C=[0 0 1];
D=0;
n=3;
Beschichtungsanlage=ss(A, B, C, D);
