%   Script MOTOR.M
%
%  Gleichstrommotor
%
% J. Lunze
% 15.11.2002
% fr 2. Auflage: 27.12.2006
% fr 3. Auflage: 17.1.2012
% fr 5. Auflage: 18.11.2019
%
%
clear
close all
%                   Motorparameter
%
R=0.5;         % Ohm
J=13E-4;        % Nms^2/rad
kM=30E-3;       % Vs/rad
kT=30E-3;       % Nm/A
kR=30E-6;        % Nms/rad
L=5E-2;        % H
n=2;
A=[-R/L  -kM/L;
    kT/J   -kR/J];
B=[1/L; 0];
C=[0 1];
D=0;
Motor=ss(A, B, C, D);
%printsys(A, B, C, D);


%%   Statisches Verhalten des Motors
%    bei Erregung mit 12 Volt
%
x0 = - inv(A)*B*12
ks=dcgain(Motor)
ybar=dcgain(Motor)*12


%%   Eigenbewegung im Zustandsraum
%
figure(1)
subplot(3,1,[1,2]);
%         Erregung mit 12 V
%         statischer Zustand
[Y, T, X]=initial(Motor, x0);
plot(X(:,1), X(:,2), 'b', [-100 100], [0 0], 'k:', [0 0], [-500 500], 'k:', x0(1), x0(2), 'bo')
axis([-20 5 -50 500]);
yticks([0 200  400]);
xticks([-20 -10 0]);
latextext(0.8, 420, '$$\mathbf{x}_0$$');
latextext(-3, 420, '$$\leftarrow$$');
latexxlabel('$$i$$ in A');
latexylabel('$$\omega$$ in $$\frac{\rm rad}{\rm s}$$');
rotateY;
latextitle('Eigenbewegung des Motors');


%% bergangsfunktion des Motors
%
figure(2)
subplot(4,1,[1,2])
[Y, T]=step(Motor);
plot(T, Y, 'b', [-0.2 0], [0 0], 'b',  [0 10], [ks ks], 'k:');
axis([-0.2 2.5 -2 35]);
yticks([0 10 20 30]);
xticks([0 1 2 3]);
latextitle('\"Ubergangsfunktion des Fensterhebers');
latexxlabel('$$t$$ in s');
latexylabel('$$\omega$$ in $$\frac{\rm rad}{\rm s}$$');
rotateY;
latextext(2.6, ks, '$$k_{\rm s}$$');



%%     Bilder in .EPS-Dateien
Epsbildc15('motor');
